@@ -42,6 +42,18 @@ typedef enum{
   AGS_SYNTH_GENERATOR_OSCILLATOR_SQUARE,
 }AgsSynthGeneratorOscillator;
 
+typedef enum{
+  AGS_SYNTH_GENERATOR_COMPUTE_FIXED_LENGTH  = 1,
+  AGS_SYNTH_GENERATOR_COMPUTE_SYNC          = 1 <<  1,
+  AGS_SYNTH_GENERATOR_COMPUTE_16HZ          = 1 <<  2,
+  AGS_SYNTH_GENERATOR_COMPUTE_440HZ         = 1 <<  3,
+  AGS_SYNTH_GENERATOR_COMPUTE_22000HZ       = 1 <<  4,
+  AGS_SYNTH_GENERATOR_COMPUTE_LIMIT         = 1 <<  5,
+  AGS_SYNTH_GENERATOR_COMPUTE_NOHZ          = 1 <<  6,
+  AGS_SYNTH_GENERATOR_COMPUTE_FREQUENCY     = 1 <<  7,
+  AGS_SYNTH_GENERATOR_COMPUTE_NOTE          = 1 <<  8,
+}AgsSynthGeneratorComputeFlags;
+
 struct _AgsSynthGenerator
 {
   AgsFunction function;
@@ -53,6 +65,7 @@ struct _AgsSynthGenerator
   guint format;
 
   guint n_frames;
+  guint attack;
   
   guint oscillator;
   
@@ -70,6 +83,14 @@ struct _AgsSynthGeneratorClass
   
 GType ags_synth_generator_get_type();
 
+GObject* ags_synth_generator_compute(AgsSynthGenerator *synth_generator,
+				     gdouble note);
+void ags_synth_generator_compute_with_audio_signal(AgsSynthGenerator *synth_generator,
+						   GObject *audio_signal,
+						   gdouble note,
+						   AgsComplex *sync_start, AgsComplex *sync_end,
+						   guint compute_flags);
+
 AgsSynthGenerator* ags_synth_generator_new();
 
 #endif /*__AGS_SYNTH_GENERATOR_H__*/
